@@ -14,3 +14,6 @@
 #undef HAVE_SOLARIS_XINERAMA
 #undef HAVE_XFREE_XINERAMA
 #undef HAVE_SHAPE
+#undef HAVE_RANDR
+#undef HAVE_STARTUP_NOTIFICATION
+
