@@ -19,9 +19,11 @@
  * 02111-1307, USA.
  */
 
+#include <config.h>
 #include "core.h"
 #include "frame.h"
 #include "workspace.h"
+#include "prefs.h"
 
 void
 meta_core_get_client_size (Display *xdisplay,
@@ -216,6 +218,22 @@ meta_core_user_raise (Display *xdisplay,
   meta_window_raise (window);
 }
 
+void
+meta_core_user_lower (Display *xdisplay,
+                      Window   frame_xwindow)
+{
+  MetaDisplay *display;
+  MetaWindow *window;
+  
+  display = meta_display_for_x_display (xdisplay);
+  window = meta_display_lookup_x_window (display, frame_xwindow);
+
+  if (window == NULL || window->frame == NULL)
+    meta_bug ("No such frame window 0x%lx!\n", frame_xwindow);
+  
+  meta_window_lower (window);
+}
+
 void
 meta_core_user_focus (Display *xdisplay,
                       Window   frame_xwindow,
@@ -305,6 +323,25 @@ meta_core_maximize (Display *xdisplay,
   meta_window_maximize (window);
 }
 
+void
+meta_core_toggle_maximize (Display *xdisplay,
+                           Window   frame_xwindow)
+{
+  MetaDisplay *display;
+  MetaWindow *window;
+  
+  display = meta_display_for_x_display (xdisplay);
+  window = meta_display_lookup_x_window (display, frame_xwindow);
+
+  if (window == NULL || window->frame == NULL)
+    meta_bug ("No such frame window 0x%lx!\n", frame_xwindow);  
+
+  if (window->maximized)
+    meta_window_unmaximize (window);
+  else
+    meta_window_maximize (window);
+}
+
 void
 meta_core_unmaximize (Display *xdisplay,
                       Window   frame_xwindow)
@@ -417,9 +454,8 @@ meta_core_change_workspace (Display *xdisplay,
     meta_bug ("No such frame window 0x%lx!\n", frame_xwindow);  
 
   meta_window_change_workspace (window,
-                                meta_display_get_workspace_by_screen_index (display,
-                                                                            window->screen,
-                                                                            new_workspace));
+                                meta_screen_get_workspace_by_index (window->screen,
+                                                                    new_workspace));
 }
 
 int
@@ -439,7 +475,7 @@ meta_core_get_active_workspace (Screen *xscreen)
 
   screen = meta_screen_for_x_screen (xscreen);
 
-  return meta_workspace_screen_index (screen->active_workspace);
+  return meta_workspace_index (screen->active_workspace);
 }
 
 int
@@ -503,18 +539,121 @@ meta_core_show_window_menu (Display *xdisplay,
   if (window == NULL || window->frame == NULL)
     meta_bug ("No such frame window 0x%lx!\n", frame_xwindow);  
 
+  meta_window_raise (window);
+  meta_window_focus (window, timestamp);
+
   meta_window_show_menu (window, root_x, root_y, button, timestamp);
 }
 
+void
+meta_core_get_menu_accelerator (MetaMenuOp           menu_op,
+                                int                  workspace,
+                                unsigned int        *keysym,
+                                MetaVirtualModifier *modifiers)
+{
+  const char *name;
+
+  name = NULL;
+  
+  switch (menu_op)
+    {
+    case META_MENU_OP_DELETE:
+      name = META_KEYBINDING_CLOSE;
+      break;
+    case META_MENU_OP_MINIMIZE:
+      name = META_KEYBINDING_MINIMIZE;
+      break;
+    case META_MENU_OP_UNMAXIMIZE:
+      name = META_KEYBINDING_UNMAXIMIZE;
+      break;
+    case META_MENU_OP_MAXIMIZE:
+      name = META_KEYBINDING_MAXIMIZE;
+      break;
+    case META_MENU_OP_UNSHADE:
+      name = META_KEYBINDING_TOGGLE_SHADE;
+      break;
+    case META_MENU_OP_SHADE:
+      name = META_KEYBINDING_TOGGLE_SHADE;
+      break;
+    case META_MENU_OP_UNSTICK:
+      name = META_KEYBINDING_TOGGLE_STICKY;
+      break;
+    case META_MENU_OP_STICK:
+      name = META_KEYBINDING_TOGGLE_STICKY;
+      break;
+    case META_MENU_OP_WORKSPACES:
+      switch (workspace)
+        {
+        case 1:
+          name = META_KEYBINDING_MOVE_WORKSPACE_1;
+          break;
+        case 2:
+          name = META_KEYBINDING_MOVE_WORKSPACE_2;
+          break;
+        case 3:
+          name = META_KEYBINDING_MOVE_WORKSPACE_3;
+          break; 
+        case 4:
+          name = META_KEYBINDING_MOVE_WORKSPACE_4;
+          break; 
+        case 5:
+          name = META_KEYBINDING_MOVE_WORKSPACE_5;
+          break; 
+        case 6:
+          name = META_KEYBINDING_MOVE_WORKSPACE_6;
+          break; 
+        case 7:
+          name = META_KEYBINDING_MOVE_WORKSPACE_7;
+          break; 
+        case 8:
+          name = META_KEYBINDING_MOVE_WORKSPACE_8;
+          break; 
+        case 9:
+          name = META_KEYBINDING_MOVE_WORKSPACE_9;
+          break; 
+        case 10:
+          name = META_KEYBINDING_MOVE_WORKSPACE_10;
+          break;
+        case 11:
+          name = META_KEYBINDING_MOVE_WORKSPACE_11;
+          break;
+        case 12:
+          name = META_KEYBINDING_MOVE_WORKSPACE_12;
+          break;
+        }
+      break;
+    case META_MENU_OP_MOVE:
+      name = META_KEYBINDING_BEGIN_MOVE;
+      break;
+    case META_MENU_OP_RESIZE:
+      name = META_KEYBINDING_BEGIN_RESIZE;
+      break;
+    }
+
+  if (name)
+    {
+      meta_prefs_get_window_binding (name, keysym, modifiers);
+    }
+  else
+    {
+      *keysym = 0;
+      *modifiers = 0;
+    }
+}
+
 char *
 meta_core_get_workspace_name_with_index (Display *xdisplay,
-                                         int index)
+                                         Window   xroot,
+                                         int      index)
 {
   MetaDisplay *display;
+  MetaScreen *screen;
   MetaWorkspace *workspace;
 
   display = meta_display_for_x_display (xdisplay);
-  workspace = meta_display_get_workspace_by_index (display, index);
+  screen = meta_display_screen_for_root (display, xroot);
+  g_assert (screen != NULL);
+  workspace = meta_screen_get_workspace_by_index (screen, index);
   return (workspace != NULL) ? workspace->name : NULL;
 }
 
@@ -629,7 +768,7 @@ meta_core_set_screen_cursor (Display *xdisplay,
   if (window == NULL || window->frame == NULL)
     meta_bug ("No such frame window 0x%lx!\n", frame_on_screen);  
 
-  meta_screen_set_cursor (window->screen, cursor);
+  meta_frame_set_screen_cursor (window->frame, cursor);
 }
 
 void
