@@ -64,6 +64,8 @@ struct _MetaScreen
   
   MetaWorkspace *active_workspace;
 
+  GList *workspaces;
+  
   MetaStack *stack;
 
   MetaCursor current_cursor;
@@ -77,6 +79,12 @@ struct _MetaScreen
 
   /* Cache the current Xinerama */
   int last_xinerama_index;
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+  SnMonitorContext *sn_context;
+  GSList *startup_sequences;
+  guint startup_sequence_timeout;
+#endif
   
   guint work_area_idle;
 
@@ -87,7 +95,8 @@ struct _MetaScreen
   
   guint keys_grabbed : 1;
   guint all_keys_grabbed : 1;
-
+  guint showing_desktop : 1;
+  
   int closing;
 };
 
@@ -104,6 +113,9 @@ void          meta_screen_queue_window_resizes (MetaScreen                 *scre
 
 int           meta_screen_get_n_workspaces    (MetaScreen                 *screen);
 
+MetaWorkspace* meta_screen_get_workspace_by_index (MetaScreen    *screen,
+                                                   int            index);
+
 void          meta_screen_set_cursor          (MetaScreen                 *screen,
                                                MetaCursor                  cursor);
 
@@ -136,4 +148,9 @@ void meta_screen_resize (MetaScreen *screen,
                          int         height);
 
 
+/* Show/hide the desktop (temporarily hide all windows) */
+void     meta_screen_show_desktop   (MetaScreen *screen);
+void     meta_screen_unshow_desktop (MetaScreen *screen);
+
+
 #endif
