@@ -24,12 +24,25 @@
 
 #include "window.h"
 
+/* Negative to avoid conflicting with real workspace
+ * numbers
+ */
+typedef enum
+{
+  META_MOTION_UP = -1,
+  META_MOTION_DOWN = -2,
+  META_MOTION_LEFT = -3,
+  META_MOTION_RIGHT = -4
+} MetaMotionDirection;
+
 struct _MetaWorkspace
 {
   MetaScreen *screen;
   
   GList *windows;
 
+  char *name;
+  
   MetaRectangle work_area;
   guint work_area_invalid : 1;
 };
@@ -47,7 +60,6 @@ gboolean       meta_workspace_contains_window (MetaWorkspace *workspace,
                                                MetaWindow  *window);
 void           meta_workspace_activate      (MetaWorkspace *workspace);
 int            meta_workspace_index         (MetaWorkspace *workspace);
-int            meta_workspace_screen_index  (MetaWorkspace *workspace);
 GList*         meta_workspace_list_windows  (MetaWorkspace *workspace);
 
 void meta_workspace_invalidate_work_area (MetaWorkspace *workspace);
@@ -55,6 +67,12 @@ void meta_workspace_get_work_area        (MetaWorkspace *workspace,
                                           MetaRectangle *area);
 
 
+MetaWorkspace* meta_workspace_get_neighbor (MetaWorkspace      *workspace,
+                                            MetaMotionDirection direction);
+
+void meta_workspace_set_name (MetaWorkspace *workspace,
+                              const char    *name);
+
 #endif
 
 
